/*
 * Synthesizer.asm
 *
 *  (c) Samuel Kleiser 2011
 *   
 * Not for commercial use
 */ 

.NOLIST
.include "m8def.inc"	
.LIST
.include "config.asm"
.include "midii.asm"
		

.def nullreg	= r2
.def nullreg2	= r3
.def Ztemp		= r4
.def Ztemp2		= r5
.def temp6	 	= r6
.def Bufwp		= r7
.def hrfreq		= r8
.def hrfreq2	= r9
.def hrfreq3	= r10
.def hrfreq4	= r11
.def hrfreq5	= r12
.def hrfreq6	= r13
.def sample 	= r14
.def sample2	= r15
.def temp 		= r16
.def temp2		= r17
.def presc		= r18
.def timer		= r19
.def volsum		= r20
.def volsum2	= r21
.def temp3		= r22
.def temp4		= r23
.def temp5		= r24
.def status 	= r25

;Statuswerte
.equ ST_WAIT	= 0x05
.equ ST_END 	= 0x06

;OSC-Daten:
;
;Aufbau:	
;---Variable Werte---------
;	1		Status
;	2-3		Frequenzzhler
;	5		Klang
;	6		Wiederholungszhler
;---Konstante Werte--------
;	7-8		Frequenz
;	9		Volume
;	13		Dauer bis zum nchsten Befehl in Ticks / 1024
;	14-15	Spurposition der Mididatei
;	16-17	Startposition der Mididatei
;--------------------------

;******************************
;Oscillatorstruktur
.equ state 		= 0
.equ freqc 		= 1
.equ sound		= 3
.equ repc		= 4
.equ freq		= 5
.equ vol		= 7
.equ ADSRcnt	= 8
.equ VolVibcnt	= 9
.equ ToneDur	= 10
.equ mpos 		= 11;16bit
.equ smpos		= 13;16bit

.equ OSCLEN 	= 15
;******************************

;******************************
;Hllkurvenstrukur
.equ AttackV 	= 0
.equ AttackS	= 1
.equ DecayV		= 2
.equ DecayS		= 3
.equ VolVibV	= 4
.equ VolVibS	= 5
.equ ReleaseS	= 6
.equ ReleaseEnd	= 7	;16bit

.equ ADSRlen	= 9
;******************************

;******************************
;Samplebuffer

.equ MR_IT = 64
.equ BUFLEN = 256
.equ T1_PRESC = 128 / MR_IT
.equ T2_PRESC = (Speed / MR_IT / T1_PRESC)


.if MR_IT > BUFLEN / 2
	.error HR_IT darf hoechstens halb so gross wie BUFLEN sein!
.endif
.if BUFLEN != 256
	.error BUFLEN muss 256 sein!
.endif
.if BUFLEN / MR_IT * MR_IT != BUFLEN
 .error HR_IT muss eine 2^x als Basis haben!
.endif

;******************************



.org 0x000
	rjmp INIT

;****************************************************
;*				Gepufferte PWM-Ausgabe				*
;*						 		 					*
;*	Laufzeit	  @8bitPWM:	15 Takte				*
;*	CPU-Belastung @8bitPWM:	 6,5 %					*
;*													*
;*	Verwendete Register:	X-Pointer				*
;*													*
;****************************************************

;ACHTUNG!!! DIESE ISR DARF SREG NICHT VERNDERN!!! SREG WIRD NICHT GESICHERT!

;Timer0 OVF Interrupt Handler: Gepufferte Ausgabe der Samples per PWM
.org OC1Aaddr					; 4	:
	ldi XH, HIGH(BUF)			; 1 : 
.if F_CPU / 512 >= 44100		;F_CPU >= 22,050Mhz
.warning "9bit PWM"
	push temp
	push temp2
	ld temp2, X+				;highbyte laden
	ld temp, X+					;lowbyte laden
	in XH, SREG					;Sreg sichern
	lsl temp					;Vorderste 9bit des 16bit Samples in das 16bit PWM Register schreiben = 7 nach rechts schieben
	rol temp2
	clr temp
	rol temp
	out OCR1AH, temp
	out OCR1AL, temp2
	out SREG, XH
	pop temp2
	pop temp
.elif F_CPU / 256 >= 44100		;F_CPU >= 11,025Mhz
.warning "8bit PWM"
	ld XH, X					; 2	: nchstes Byte aus Ringpuffer lesen
	out OCR1AL, XH				; 1	:
	in XH, SREG					; 1 :
	subi XL, -2					; 1 :
	out SREG, XH				; 1 :
.elif F_CPU / 128 >= 44100		;F_CPU >= 5,525Mhz
.warning "7bit PWM"
	push temp
	ld XH, X
	in temp, SREG
	lsr XH						;Sample durch 2 teilen (7bit PWM)
	out OCR1AL, XH
	subi XL, -2
	out SREG, temp
	pop temp
.elif F_CPU / 64 >= 44100
.warning "6bit PWM"
	push temp
	ld XH, X
	in temp, SREG
	lsr XH						;Sample durch 4 teilen (6bit PWM)
	lsr XH
	out OCR1AL, XH
	subi XL, -2
	out SREG, temp
	pop temp
.else
	.error "F_CPU zu niedrig!"
.endif
	

	reti						; 4	:							
								;---
								;13	:

;*****************************************************


;Init
INIT:
	ldi temp, HIGH(RAMEND) 		;Stackpointer initialisieren
	out SPH, temp
	ldi temp, LOW(RAMEND)
	out SPL, temp

	clr nullreg			;Nullregister auf 0 setzen
	clr nullreg2		;Spter zum zucksetzen von Register wichtig

	ldi temp, 1<<PB1
	out DDRB, temp

	;PWM initialisieren
	ldi temp, (1<<COM1A1)| (1<<WGM11)
	out TCCR1A, temp
	ldi temp, (1<<CS10) | (1<<WGM12) | (1<<WGM13);Einstellbare Obergrenze (ICR1), kein Prescaler
	out TCCR1B, temp
	ldi temp2, HIGH(F_CPU/44100-1)				;Obergrenze fr 44100kHz berechnen
	ldi temp, LOW(F_CPU/44100-1)
	out ICR1H, temp2
	out ICR1L, temp

	ldi temp, 0xFF
	out OCR1AL, temp
	ldi temp, 1<<OCIE1A
	out TIMSK, temp								;Nach jedem PWM-Durchlauf, PWM-Wert ndern -> Comparematch Interrupt

	;Pointer fr Buffer setzen
	ldi XH, HIGH(BUF)			;X-Pointer mit Startadresse
	ldi XL, LOW(BUF) 			;des Buffers laden
	mov bufwp, XL				;Bufferschreibposition sichern

    ldi YH, HIGH(BUF)
    ldi YL, LOW(BUF)

	ldi YH, HIGH(OSC)			;OSC-Startadresse in Y-Pointer laden
	ldi YL, LOW(OSC)

	ldi timer, T2_PRESC			;2 Software Timer
	ldi presc, T1_PRESC

	sei
	
	;OSC-Daten initialisieren
    ldi ZH, HIGH(MIDI*2)		;Mididateiadresse in Z-Pointer laden
	ldi ZL, LOW(MIDI*2)
	lpm temp5, Z+
	adiw Z, 1
L1:
	ldi temp, ST_WAIT			;State auf Startstate setzen
	std Y+state, temp
	ldi temp, 255 / OSCCNT		;Lautstrke auf sicheren Wert setzen: Alle Kanle zusammen drfen hchstens die Lautstrke 255 haben, sonst bersteuert es
	std Y+vol, temp
	std Y+sound, nullreg
	std Y+repc, nullreg
	std Y+ADSRcnt, nullreg

	ldi temp3, 1
	std Y+VolVibcnt, nullreg
	std Y+ToneDur, temp3		;auf 1 setzen, damit bald der nchste Ton geladen wird

	lpm temp, Z+				;Spurstartadressen auslesen
	lpm temp2, Z+				;dessen Highbyte lesen

	subi temp, LOW(-SMIDI*2)	;Dateianfang addieren
	sbci temp2, HIGH(-SMIDI*2)
	std Y+mpos, temp			;In OSC-Struktur speichern
	std Y+(mpos+1), temp2		;Highbyte speichern
	std Y+smpos, temp			;Startadresse speichern
	std Y+(smpos+1), temp2

	subi YL, -OSCLEN			;Im SRAM zur nchsten Adresse zum speichern gehen
	dec temp5
	brne L1						;Nein? Nchste Adresse speichern

	ldi status, ST_END
	rjmp LCOMP
LX:
	std Y+state, status
	subi YL, -OSCLEN
LCOMP:
	cpi YL, OSCCNT*OSCLEN
	brne LX

	ldi YH, HIGH(ADSR)
	ldi YL, LOW(ADSR)
L2:
	ldi ZH, HIGH(SOUNDS2*2)
	ldi ZL, LOW(SOUNDS2*2)
	L3:
		lpm temp, Z+
		st Y+, temp
		cpi ZL, ADSRlen
		brne L3
	subi YL, ADSRlen

	subi YL, -ADSRlen
	cpi YL, LOW(OSCCNT*ADSRlen)
	brne L2
	

InitLoop:
	ldi YH, HIGH(OSC)			;OSC-Pointer setzen
	ldi YL, LOW(OSC)
	movw sample:sample2, nullreg:nullreg2	;Sample lschen
	rjmp MR_Begin

;********************************************************************************
;*****************************Hauptroutine***************************************
;********************************************************************************

;Laderoutine fr die "Midi"-Daten

M_TABLE:						;Sprungtabelle
	rjmp M_END
	rjmp M_TONE
	rjmp M_DELAY
	rjmp M_VOL
	rjmp M_JMP
	rjmp M_SOUND

M_TONE:
	movw Z, Ztemp:Ztemp2		;Z-Pointer wiederherstellen
	lpm temp, Z+				;8bit-freqindex aus dem Flash lesen und in ZL speichern
	lpm temp2, Z+				;Neue Befehlsdauer lesen
	std Y+ToneDur, temp2
	
	;neue Leseposition speichern
	std Y+(mpos+1), ZH
	std Y+mpos, ZL

	lsl temp					;*2 (Wordtabelle); mglich da es nur 128 Tne in der Tabelle sind (kein weiteres rol ntig)
	ldi ZH, HIGH(TONES*2)		;Z-Pointer mit Tabellenstart fllen
	ldi ZL, LOW(TONES*2)
	add ZL, temp				;Index addieren
	adc ZH, nullreg				;Carry addieren
	lpm temp, Z+				;Frequenzdaten laden
	lpm temp2, Z+

	std Y+freq, temp
	std Y+(freq+1), temp2
	std Y+state, nullreg		;Status auf 0 setzen (Attack)
	;Zhlregister zurcksetzen
	std Y+freqc, nullreg		;16bit Frequenzzhler auf 0 setzen
	std Y+freqc+1, nullreg
	std Y+ADSRcnt, nullreg

	rjmp TEL_NEXT

M_DELAY:
	ldi status, ST_WAIT
	std Y+state, status

	movw Z, Ztemp:Ztemp2		;Z-Pointer wiederherstellen
	lpm temp, Z+				;Dauer laden
	std Y+ToneDur, temp			;Dauer speichern
	;neue Leseposition speichern
	std Y+(mpos+1), ZH
	std Y+mpos, ZL
	rjmp TEL_NEXT

M_VOL:
	movw Z, Ztemp:Ztemp2		;Z-Pointer wiederherstellen
	lpm temp, Z+
	std Y+vol, temp

	rjmp LoadID					;nchsten Befehl laden

M_JMP:
	movw Z, Ztemp:Ztemp2
	lpm temp, Z+				;Sprungposition laden (low)
	ldd temp2, Y+smpos			;Spurstart (low) laden
	add temp, temp2				;Zu Spurstart addieren
	lpm temp2, Z+				;highbyte laden
	ldd ZL, Y+(smpos+1)			;Spurstart (high) landen
	adc temp2, ZL				;highbytes mit carry addieren
	std Y+mpos, temp			;Neue Spurposition speichern 
	std Y+(mpos+1), temp2
	movw Z, temp:temp2			;Z-Pointer auch auf neue Position setzen,
	rjmp LoadID					;da der nchste Befehl sofort geladen wird

M_SOUND:
	movw Z, Ztemp:Ztemp2
	lpm temp, Z+
	movw Ztemp:Ztemp2, Z
	std Y+sound, temp
	;spezifische Hllkurve auslesen
	ldi temp2, ADSRlen			;Index mit Datenlnge multiplizieren
	mul temp, temp2
	mov ZL, r0
	ldi ZH, HIGH(SOUNDS2*2)		;ZH laden
	add r0, temp2				;Endwert fr Schleife berechnen (r0=ZL, temp2=ADSRdata)
	movw temp:temp2, Y
	movw Y, temp3:temp4			;ADSR-Pointer laden
	;Werte laden und im Ram speichern
M_SOUND1:
	lpm r1, Z+
	st Y+, r1
	cpse ZL, r0					;Auf Ende prfen (ZL == ADSRdata)
	rjmp M_SOUND1

	movw Y, temp:temp2			;Y wiederherstellen
	movw Z, Ztemp:Ztemp2		;Z wiederherstellen

	rjmp LoadID					;nchsten Befehl laden

;Hllkurvenerzeugen:
;
;	 ________________________
;	|                        |
;	|      / \               |
;	|    /     \ _ _ _       |
;	|  /               \     |
;	|/ Attack   Sustain  \   |
;	|       Decay    Release |
;	|________________________|
;	
;	y = temp4	x = status
;
	
		H_TABLE:
			rjmp H_ATTACK
			rjmp H_DECAY
			rjmp H_SUSTAIN
			rjmp H_SUSTAIN_END
			rjmp H_RELEASE
			rjmp H_END
			rjmp H_END

		H_ATTACK:
			movw Z, temp:temp2
			ldd temp3, Y+AttackS	;Attackspeed laden
			add temp4, temp3		;Lautstrke mit Wert erhhen
			brcs H_DECAY_START_C
			ldd temp3, Y+AttackV	;Maximaler Attackwert laden
			cp temp4, temp3			;Prfen ob Attack fertig ist
			brsh H_DECAY_START
			std Z+ADSRcnt, temp4	;Hllkurve speichern - der maximale Attackwert wird nie erreicht, da vorher zu DECAY gesprungen wird
			rjmp H_END
		H_DECAY_START_C:
			ser temp3				;Attackwert berlief, auf Maximum setzen
		H_DECAY_START:
			inc status
			std Z+state, status
			std Z+ADSRcnt, temp3	;Attackwert wurde berschritten, auf Wunschwert setzen
			rjmp H_END
		H_DECAY:
			movw Z, temp:temp2
			ldd temp3, Y+DecayS		;Decayspeed laden
			sub temp4, temp3		;Lautstrke erniedrigen
			ldd temp3, Y+DecayV		;Endwert laden
			brcs H_SUSTAIN_START
			cp temp4, temp3
			brlo H_SUSTAIN_START
			std Z+ADSRcnt, temp4	;Hllkurve speichern
			rjmp H_END
		H_SUSTAIN_END_START:
			inc status
			std Z+state, status
		H_SUSTAIN_END:
			ldd temp3, Y+(ReleaseEnd+1)	;Timerzahl laden bei der Release beginnt
			cpse temp3, timer		;Stimmen die Zahlen berein
			rjmp H_END				;Nein? Sustain noch nicht zuende
			movw Z, temp:temp2
			inc status				;Ja? Release einleiten
			std Z+state, status;Status erhhen
		H_RELEASE:
			movw Z, temp:temp2
			ldd temp3, Y+ReleaseS
			sub temp4, temp3
			std Z+ADSRcnt, temp4
			brcc H_END
			inc status
			std Z+state, status
			rjmp H_END
		H_SUSTAIN_START:
			inc status
			std Z+state, status		;Neuen Status (Sustain) speichern
			std Z+ADSRcnt, temp3	;Endwert speichern, da er unterschritten wurde
		H_SUSTAIN:					;6
			ldd temp3, Y+ReleaseEnd	;Duration laden, bei der Release beginnt
			ldd temp4, Y+ToneDur	;Aktuelle Dauer des Tones laden
			cp temp3, temp4
			breq H_SUSTAIN_END_START
		H_END:
		subi YL, -ADSRlen				;nchster Kanal
		subi temp, -OSClen
		cpi YL, LOW(ADSREND)			;auf letzen prfen
		brne YBOL

	rjmp PRESC_EQ_ret

LoadI:							;nchsten Befehl laden:
	ldd ZH, Y+(mpos+1)			;Adresse fr nchsten Befehl in Z-Pointer
	ldd ZL, Y+mpos				;laden
LoadID:
	lpm temp, Z+				;nchsten Befehl laden, Z inkreminieren, damit spter weitergeladen werden kann
	
	movw Ztemp:Ztemp2, Z		;Z-Pointer sichern
	
	cpi temp, ICNT				;Befehl vorhanden
	brsh M_END					;Index zu gro
	ldi ZH, HIGH(M_TABLE)		;Z-Pointer laden
	ldi ZL, LOW(M_TABLE)
	add ZL, temp
	adc ZH, nullreg
	ijmp

M_END:
	ldi status, ST_END
	std Y+state, status
	rjmp TEL_NEXT

TIMER_EQ:
	ldi YH, HIGH(OSC)			;ADSR-Pointer setzen
	ldi YL, LOW(OSC)
	ldi temp3, LOW(ADSR)
	ldi temp4, HIGH(ADSR)
		
	TEL:
		ldd status, Y+state
		cpi status, ST_END		;Wenn die Spur zu Ende ist
		breq TEL_NEXT			;nchste Spur berechnen
		ldd temp, Y+ToneDur		;Befehlsdauer aus SRAM laden
		dec temp
		breq LoadI				;Wenn 0, neuen Befehl laden
		std Y+ToneDur, temp		;Ansonsten neue Dauer speichern
	TEL_NEXT:
		subi YL, -OSClen		;nchster Kanal
		subi temp3, -ADSRlen		

		cpi YL, LOW(OSCEND)	;auf letzen prfen
		brne TEL		
	ldi timer, T2_PRESC
	rjmp TIMER_EQ_ret

PRESC_EQ:
	ldi presc, T1_PRESC 		;Prescaler setzen
	dec timer                   ;2. soft Timer erniedrigen
	breq TIMER_EQ               ;auf Unterlauf prfen
TIMER_EQ_ret:
	ldi YH, HIGH(ADSR)			;ADSR-Pointer setzen
	ldi YL, LOW(ADSR)
	ldi ZH, HIGH(H_TABLE)		;Tabellepointer laden
	ldi ZL, LOW(H_TABLE)
	movw Ztemp:Ztemp2, Z
	ldi temp, LOW(OSC)
	ldi temp2, HIGH(OSC)
	YBOL:
		;Hllkurven berechnen
		movw Z, temp:temp2
		ldd status, Z+state	;Status: beschreibt Position der ADSR-Hllkurve
		ldd temp4,  Z+ADSRcnt	;aktuelle Lautstrke welche die Hllkurve bestimmt
		movw Z, Ztemp:Ztemp2	;Z wiederherstellen
		add ZL, status			;Status dazuaddieren
		adc ZH, nullreg			;Carry
		ijmp					;in Tabelle (oben) springen


;*********************************************************
;Synthesizer Hauptroutine: Tonerzeugung
;Optimiert durch Nutzung aller Register und Aufrollen von Schleifen und Bedingungen
;Deswegen leider schlecht lesbar, aber dafr eine unglaubliche Performance
;
;Programmablauf:
;1. Warten bis gengend Platz im Buffer ist
;2. 3 aktive Kanle in die Register laden
;3. Wenn es weniger als 3 aktive Kanle gibt, alle Aktive laden
;4. MR_IT Samples berechnen und Buffer damit berschreiben
;5. Genderte Daten sichern (SRAM) 
;6. Sollten weniger als 3 Kanle berechnet worden sein, Routine beenden
;7. Schritt 2 wiederholen, ab sofort bei Schritt 4 die Sample zu dem Buffer addieren, statt ihn zu berschreiben
;
;Weitere mgliche Optimierungen
;Warteschleife kann vor Endwert verlassen werden
;
;

;Macros
.macro MR_LoadData1
	ldd r0, Z+ADSRcnt
	ldd r1, Z+vol
	mul r0, r1
	mov volsum, r1
	ldd Ztemp, Z+freqc		;16bit Frequenzzhler laden
	ldd temp, Z+(freqc+1)
	ldd temp2, Z+sound
	subi temp2, -HIGH(Sounds*2)
	ldd hrfreq, Z+freq			;16bit Schritt laden
	ldd hrfreq2, Z+(freq+1)		;gleiche mit dem Highbyte
.endm
.macro MR_LoadData2
	ldd r0, Z+ADSRcnt
	ldd r1, Z+vol 
	mul r0, r1
	mov volsum2, r1
	ldd Ztemp2, Z+freqc		;16bit Frequenzzhler laden
	ldd temp3, Z+(freqc+1)
	ldd temp4, Z+sound
	subi temp4, -HIGH(Sounds*2)
	ldd hrfreq3, Z+freq			;16bit Schritt laden
	ldd hrfreq4, Z+(freq+1)		;gleiche mit dem Highbyte
.endm
.macro MR_LoadData3
	ldd r0, Z+ADSRcnt
	ldd r1, Z+vol
	mul r0, r1
	mov timer, r1
	ldd temp6, Z+freqc			;16bit Frequenzzhler laden
	ldd temp5, Z+(freqc+1)
	ldd status, Z+sound
	subi status, -HIGH(Sounds*2)
	ldd hrfreq5, Z+freq			;16bit Schritt laden
	ldd hrfreq6, Z+(freq+1)		;gleiche mit dem Highbyte
.endm
.macro MR_Do1
	add Ztemp, hrfreq			;zum Zhler addieren
	adc temp, hrfreq2

	;Tonerzeugung
	movw Z, temp:temp2
	lpm presc, Z

	;Lautstrke
	mulsu presc, volsum

	add sample, r0
	adc sample2, r1
.endm
.macro MR_Do2
	add Ztemp2, hrfreq3			;zum Zhler addieren
	adc temp3, hrfreq4

	;Tonerzeugung
	movw Z, temp3:temp4
	lpm presc, Z

	;Lautstrke
	mulsu presc, volsum2

	add sample, r0
	adc sample2, r1
.endm
.macro MR_Do3
	add temp6, hrfreq5			;zum Zhler addieren
	adc temp5, hrfreq6

	;Tonerzeugung
	movw Z, temp5:status
	lpm presc, Z

	;Lautstrke
	mulsu presc, timer

	add sample, r0
	adc sample2, r1
.endm
;Kein Kanal verarbeiten / Buffer leeren
MR_Start0:	
	st Y+, nullreg2
	st Y+, nullreg
	cp YL, bufwp
	brne MR_Start0
	rjmp MR_End
;Initialisierung
MR_Begin:
	dec presc
	breq PRESC_EQ
PRESC_EQ_ret:
	ldi ZH, HIGH(OSC)			;OSC-Pointer setzen
	ldi ZL, LOW(OSC)
	push timer					;Neue freie Variable im mulsu Bereich
	push presc
	ldi YL, 0x80
	mov nullreg2, YL
	mov YL, bufwp
	ldi YH,  MR_IT
	add bufwp, YH
	ldi YH, HIGH(BUF)
;Warten bis Platz im Buffer ist
MR_Wait:
	mov status, XL
	sub status, YL
	cpi status, MR_IT
	brlo MR_Wait				;Warten bis genug Platz im Bufffer ist
	rjmp MR_Load
;Register mit Daten laden
MR_Next1:
	subi ZL, -OSCLEN
	cpi ZL, LOW(OSCEND)
	breq MR_Start0
	;Prfen
MR_Load:
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_Next1				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	MR_LoadData1
MR_Next2:
	subi ZL, -OSCLEN
	cpi ZL, LOW(OSCEND)
	breq MR_Start1
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_Next2				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	MR_LoadData2
MR_Next3:
	subi ZL, -OSCLEN
	cpi ZL, LOW(OSCEND)
	breq MR_Start2
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_Next3				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	MR_LoadData3
;3 Kanle verarbeiten
	push ZL						;ZL im Stack zwischenspeichern
MR_Loop3:	
	movw sample:sample2, nullreg:nullreg2	;Altes Sample lschen	
	
	MR_Do1
	MR_Do2
	MR_Do3
	
	st Y+, sample2
	st Y+, sample
	cp YL, bufwp
	brne MR_Loop3
	
	pop ZL
	ldi ZH, HIGH(OSC)
	
	rjmp MR_Store3	
;1 Kanal verarbeiten
MR_Start1:
	push ZL						;ZL im Stack zwischenspeichern
MR_Loop1:	
	movw sample:sample2, nullreg:nullreg2	;Altes Sample lschen	

	MR_Do1
	
	st Y+, sample2
	st Y+, sample
	cp YL, bufwp
	brne MR_Loop1
	
	pop ZL
	ldi ZH, HIGH(OSC)

	clr r0

	rjmp MR_Store1
;2 Kanle verarbeiten	
MR_Start2:
	push ZL						;ZL im Stack zwischenspeichern
MR_Loop2:	
	movw sample:sample2, nullreg:nullreg2	;Altes Sample lschen	

	MR_Do1
	MR_Do2
	
	st Y+, sample2
	st Y+, sample
	cp YL, bufwp
	brne MR_Loop2

	pop ZL
	ldi ZH, HIGH(OSC)
	
	clr r0

	rjmp MR_Store2	
;Ende
MR_End:
	clr nullreg2
	pop presc
	pop timer
	rjmp MR_Begin			;Routine fertig - zum Anfang spring
;Relevante Daten in SRAM zurckschreiben
MR_Store3:						;Frequenzzhler speichern
	movw r0:r1, Z
	std Z+freqc, temp6
	std Z+freqc+1, temp5
MR_Store2:						;Wenn nur 2 Kanle geupdated werden, ist ZL == LOW(OSCEND) und muss erst erniedrigt werden!
	subi ZL, OSCLEN
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_Store2				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	std Z+freqc, Ztemp2	
	std Z+freqc+1, temp3
MR_Store1:						;siehe MR_Store2
	subi ZL, OSCLEN
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_Store1				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	std Z+freqc, Ztemp	
	std Z+freqc+1, temp

	cp r0, nullreg
	breq MR_End
	subi YL, LOW(MR_IT)
	sbci YH, HIGH(MR_IT)
	movw Z, r0:r1
;Register mit Daten laden
MR_NextA1:
	subi ZL, -OSCLEN
	cpi ZL, LOW(OSCEND)
	breq MR_End
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_NextA1				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	MR_LoadData1
MR_NextA2:
	subi ZL, -OSCLEN
	cpi ZL, LOW(OSCEND)
	breq MR_StartA1
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_NextA2				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	MR_LoadData2
MR_NextA3:
	subi ZL, -OSCLEN
	cpi ZL, LOW(OSCEND)
	breq MR_StartA2
	;Prfen
	ldd status, Z+state			;Statusbyte der Spur laden
	cpi status, ST_WAIT			;Wenn die Spur eine Pause enthlt
	brsh MR_NextA3				;nchste Spur berechnen, brsh, da die Spur auch zu Ende sein knnte
	MR_LoadData3
;3 Kanle verarbeiten und zu Buffer addieren
	push ZL						;ZL im Stack zwischenspeichern
MR_LoopA3:	
	ld sample2, Y
	ldd sample, Y+1
	
	MR_Do1
	MR_Do2
	MR_Do3
	
	st Y+, sample2
	st Y+, sample
	cp YL, bufwp
	brne MR_LoopA3
	
	pop ZL
	ldi ZH, HIGH(OSC)
	
	rjmp MR_Store3
;1 Kanal...
MR_StartA1:
	push ZL						;ZL im Stack zwischenspeichern
MR_LoopA1:	
	ld sample2, Y
	ldd sample, Y+1

	MR_Do1
	
	st Y+, sample2
	st Y+, sample
	cp YL, bufwp
	brne MR_LoopA1
	
	pop ZL
	ldi ZH, HIGH(OSC)

	clr r0

	rjmp MR_Store1
;2 Kanle...
MR_StartA2:
	push ZL						;ZL im Stack zwischenspeichern
MR_LoopA2:	
	ld sample2, Y
	ldd sample, Y+1

	MR_Do1
	MR_Do2
	
	st Y+, sample2
	st Y+, sample
	cp YL, bufwp
	brne MR_LoopA2

	pop ZL
	ldi ZH, HIGH(OSC)
	
	clr r0

	rjmp MR_Store2	






;****************************************************

;Diese Tabelle hngt sich an die nchsten x00-xFF Bytes des Flashs dran.
.include "tones.asm"
.include "sin.asm"

.include "midi.asm"
;****************************************************

.dseg							;Ram

;Sample Buffer mit PWM-Daten
.org 0x100
BUF:	.BYTE	BUFLEN
BUFEND:

;OSC-Daten:
;
;Aufbau:	
;---Variable Werte---------
;	1	Status
;	2-3	Frequenzzhler
;	4	Hllkurvenzhler

;---Konstante Werte--------
;	5-6	Frequenz
;	7	Volume
;	8	Hllkurvengeschwindigkeit 
;	9	Hllkurvenende (dur)
;	10	Hllkurvenende (timer)
;	10	Dauer bis zum nchsten Befehl in Ticks / 1024
;	11-12Spurposition der Mididatei
;	13-14Startposition der Mididatei
;--------------------------
.org 0x200
OSC:	.BYTE	OSCCNT*OSCLEN		;OSC-Daten
OSCEND:
.org 0x300
;Hllkurvendaten
ADSR:	.BYTE ADSRlen*OSCCNT
ADSREND:
